/*
 * JBoss, Home of Professional Open Source
 * Copyright 2006, JBoss Inc., and others contributors as indicated 
 * by the @authors tag. All rights reserved. 
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors. 
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but WITHOUT A 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License,
 * v.2.1 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
 * MA  02110-1301, USA.
 * 
 * (C) 2005-2006,
 * @author JBoss Inc.
 */
package org.jboss.soa.esb.samples.quickstart.jaas;

import java.io.IOException;


import javax.security.auth.callback.Callback;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

import org.jboss.soa.esb.ConfigurationException;
import org.jboss.soa.esb.actions.AbstractActionLifecycle;
import org.jboss.soa.esb.client.ServiceInvoker;
import org.jboss.soa.esb.helpers.ConfigTree;
import org.jboss.soa.esb.listeners.message.MessageDeliverException;
import org.jboss.soa.esb.message.Message;

public class JAASAction extends AbstractActionLifecycle {

	public static final String CONFIG_NAME_FAIL_TO = "fail-to";

	public static final String CONFIG_NAME_SERVICE_CATEGORY = "service-category";

	public static final String CONFIG_NAME_SERVICE_NAME = "service-name";

	public static final String CONFIG_NAME_LOGIN_MODULE_NAME = "login-module";

	private String loginModule;

	private Destination[] faildDestination;

	public JAASAction(ConfigTree config) throws ConfigurationException {

		loginModule = config.getAttribute(CONFIG_NAME_LOGIN_MODULE_NAME);

		ConfigTree[] failedDests = config.getChildren(CONFIG_NAME_FAIL_TO);

		if (failedDests == null || failedDests.length == 0) {
			throw new ConfigurationException("must define "
					+ CONFIG_NAME_FAIL_TO);
		}

		faildDestination = new Destination[failedDests.length];

		for (int i = 0; i < failedDests.length; i++) {
			ConfigTree tree = failedDests[i];
			String category = tree.getAttribute(CONFIG_NAME_SERVICE_CATEGORY);
			String name = tree.getAttribute(CONFIG_NAME_SERVICE_NAME);
			try {
				Destination failddest = new Destination(category, name);
				faildDestination[i] = failddest;

			} catch (MessageDeliverException e) {
				throw new ConfigurationException(e);
			}

		}

	}

	public Message process(Message message) throws Exception {

		if (!login(message)) {
			for (int i = 0; i < faildDestination.length; i++) {
				Destination dest = faildDestination[i];
				dest.send(message);
			}

			// stop this action pipeline
			return null;
		}

		// valid message so continue with pipeline
		return message;

	}

	private boolean login(Message message) {

		String username = (String) message.getBody().get("username");
		String password = (String) message.getBody().get("password");
		
		System.out.println("username : " + username);
		System.out.println("password : " + password);
		try {
			LoginContext lc = new LoginContext(loginModule,
					new SimpleCallbackHandler(username, password.toCharArray()));
			lc.login();
			return true;
		} catch (LoginException le) {
			return false;
		}
	}

	private class Destination {
		private ServiceInvoker invoker;

		Destination(String category, String name)
				throws MessageDeliverException {
			invoker = new ServiceInvoker(category, name);
		}

		void send(Message msg) throws MessageDeliverException {
			invoker.deliverAsync(msg);
		}
	}

	public static class SimpleCallbackHandler implements CallbackHandler {

		private String username;
		private char[] password;

		public SimpleCallbackHandler(String username, char[] password) {
			this.username = username;
			this.password = password;
		}

		public void handle(Callback[] cbs) throws IOException,
				UnsupportedCallbackException {
			for (int i = 0; i < cbs.length; i++) {
				if (cbs[i] instanceof NameCallback) {
					NameCallback nc = (NameCallback) cbs[i];
					nc.setName(username);
				} else if (cbs[i] instanceof PasswordCallback) {
					PasswordCallback pc = (PasswordCallback) cbs[i];
					pc.setPassword(password);
				} else {
					throw new UnsupportedCallbackException(cbs[i],
							"Unrecognized Callback");
				}
			}
		}

	}

}